@@ -41,7 +41,7 @@
  # OF THE POSSIBILITY OF SUCH DAMAGE.
  # ====================================================================
  #
- # $Id: apache.py,v 1.29 2001/04/11 02:52:09 gtrubetskoy Exp $
+ # $Id: apache.py,v 1.34 2001/05/25 03:27:31 gtrubetskoy Exp $
 
 import sys
 import string
@@ -58,6 +58,10 @@ import _apache
 # variable stores the last PythonPath in raw (unevaled) form.
 _path = None
 
+# this is used in Request.__init__
+def _cleanup_request(_req):
+    _req._Request = None
+
 class Request:
     """ This is a normal Python Class that can be subclassed.
         However, most of its functionality comes from a built-in
@@ -68,6 +72,11 @@ class Request:
     def __init__(self, _req):
         # look at __setattr__ if changing this line!
         self._req = _req
+        
+        # this will decrement the reference to the _req
+        # object at cleanup time. If we don't do this, we
+        # get a cirular reference and _req never gets destroyed.
+        _req.register_cleanup(_cleanup_request, _req)
 
     def __getattr__(self, attr):
         try:
@@ -92,9 +101,6 @@ class CallBack:
     A generic callback object.
     """
 
-    def __init__(self):
-        self.req = None
-
     class HStack:
         """
         The actual stack string lives in the request object so
@@ -122,11 +128,10 @@ class CallBack:
         result, handler = HTTP_INTERNAL_SERVER_ERROR, ""
 
         # is there a Request object for this request?
-        if not self.req or (self.req._req != _req):
-            # self.req is a regular oject, _req is a built-in.
-            # for speed, we mostly use the built-in. Users
-            # should only use self.req.
-            self.req = Request(_req)
+        if not _req._Request:
+            _req._Request = Request(_req)
+            
+        req = _req._Request
 
         # config
         config = _req.get_config()
@@ -177,16 +182,15 @@ class CallBack:
 
                 # find the object
                 silent = config.has_key("PythonHandlerModule")
-                object = resolve_object(self.req, module, object_str, silent)
+                object = resolve_object(req, module, object_str, silent)
 
                 if object:
 
                     # call the object
                     if config.has_key("PythonEnablePdb"):
-                        if config["PythonEnablePdb"]:
-                            result = pdb.runcall(object, self.req)
+                        result = pdb.runcall(object, req)
                     else:
-                        result = object(self.req)
+                        result = object(req)
 
                     # stop cycling through handlers
                     if result != OK:
@@ -221,7 +225,7 @@ class CallBack:
             # Program run-time error
             try:
                 (etype, value, traceback) = traceblock
-                result = self.ReportError(etype, value, traceback,
+                result = self.ReportError(req, etype, value, traceback,
                                           htype=htype, hname=handler,
                                           debug=debug)
             finally:
@@ -231,7 +235,7 @@ class CallBack:
             # Any other rerror (usually parsing)
             try:
                 exc_type, exc_value, exc_traceback = sys.exc_info()
-                result = self.ReportError(exc_type, exc_value, exc_traceback,
+                result = self.ReportError(req, exc_type, exc_value, exc_traceback,
                                           htype=htype, hname=handler, debug=debug)
             finally:
                 exc_traceback = None
@@ -239,7 +243,7 @@ class CallBack:
 	return result
 
 
-    def ReportError(self, etype, evalue, etb, htype="N/A", hname="N/A", debug=0):
+    def ReportError(self, req, etype, evalue, etb, htype="N/A", hname="N/A", debug=0):
 	""" 
 	This function is only used when debugging is on.
 	It sends the output similar to what you'd see
@@ -247,7 +251,6 @@ class CallBack:
 	"""
 
         try:
-            req = self.req._req
 
             if str(etype) == "exceptions.IOError" \
                and str(evalue)[:5] == "Write":
@@ -280,7 +283,7 @@ class CallBack:
         finally:
             # erase the traceback
             etb = None
-            return DONE
+            # we do not return anything
 
 def import_module(module_name, req=None, path=None):
     """ 
@@ -293,8 +296,9 @@ def import_module(module_name, req=None, path=None):
     autoreload, debug = 1, None
     if req:
         config = req.get_config()
-        autoreload = not config.has_key("PythonNoReload")
-        debug = not config.has_key("PythonDebug")
+        if config.has_key("PythonAutoReload"):
+            autoreload = int(config["PythonAutoReload"])
+        debug = config.has_key("PythonDebug")
 
     # try to import the module
 
@@ -304,16 +308,10 @@ def import_module(module_name, req=None, path=None):
     if  not autoreload:
 
         # import module
-        parts = string.split(module_name, '.')
-        for i in range(len(parts)):
-            f, p, d = imp.find_module(parts[i], path)
-            try:
-                mname = string.join(parts[:i+1], ".")
-                module = imp.load_module(mname, f, p, d)
-            finally:
-                if f: f.close()
-            if hasattr(module, "__path__"):
-                path = module.__path__
+        module = __import__(module_name)
+        components = string.split(module_name, '.')
+        for cmp in components[1:]:
+            module = getattr(module, cmp)
 
     else:
 
@@ -570,7 +568,7 @@ class CGIStdout(NullIO):
 
 def setup_cgi(req):
     """
-    Replace sys.stdin and stdout with an objects that reead/write to
+    Replace sys.stdin and stdout with an objects that read/write to
     the socket, as well as substitute the os.environ.
     Returns (environ, stdin, stdout) which you must save and then use
     with restore_nocgi().
